@@ -177,6 +177,9 @@ def replace_user_defined_constants(vyper_module: vy_ast.Module) -> int:
     return changed_nodes
 
 
+# TODO constant folding on log events
+
+
 def _replace(old_node, new_node):
     if isinstance(new_node, vy_ast.Constant):
         return new_node.from_node(old_node, value=new_node.value)
@@ -217,9 +220,6 @@ def replace_constant(
     for node in vyper_module.get_descendants(vy_ast.Name, {"id": id_}, reverse=True):
         parent = node.get_ancestor()
 
-        if isinstance(parent, vy_ast.Attribute):
-            # do not replace attributes
-            continue
         if isinstance(parent, vy_ast.Call) and node == parent.func:
             # do not replace calls
             continue
