@@ -1,14 +1,6 @@
 import io
 import re
-from tokenize import (
-    COMMENT,
-    NAME,
-    OP,
-    TokenError,
-    TokenInfo,
-    tokenize,
-    untokenize,
-)
+from tokenize import COMMENT, NAME, OP, TokenError, TokenInfo, tokenize, untokenize
 from typing import Tuple
 
 from semantic_version import NpmSpec, Version
@@ -38,15 +30,19 @@ def validate_version_pragma(version_str: str, start: ParserPosition) -> None:
     """
     from vyper import __version__
 
+    # NOTE: should be `x.y.z.*`
+    installed_version = ".".join(__version__.split(".")[:3])
+
     version_arr = version_str.split("@version")
 
     raw_file_version = version_arr[1].strip()
     strict_file_version = _convert_version_str(raw_file_version)
-    strict_compiler_version = Version(_convert_version_str(__version__))
+    strict_compiler_version = Version(_convert_version_str(installed_version))
 
     if len(strict_file_version) == 0:
         raise VersionException(
-            "Version specification cannot be empty", start,
+            "Version specification cannot be empty",
+            start,
         )
 
     try:
@@ -61,7 +57,7 @@ def validate_version_pragma(version_str: str, start: ParserPosition) -> None:
     if not npm_spec.match(strict_compiler_version):
         raise VersionException(
             f'Version specification "{raw_file_version}" is not compatible '
-            f'with compiler version "{__version__}"',
+            f'with compiler version "{installed_version}"',
             start,
         )
 
@@ -126,7 +122,10 @@ def pre_parse(code: str) -> Tuple[ModificationOffsets, str]:
 
             if typ == NAME and string in ("class", "yield"):
                 raise SyntaxException(
-                    f"The `{string}` keyword is not allowed. ", code, start[0], start[1],
+                    f"The `{string}` keyword is not allowed. ",
+                    code,
+                    start[0],
+                    start[1],
                 )
 
             if typ == NAME and string == "contract" and start[1] == 0:
