@@ -2,10 +2,7 @@ import copy
 import textwrap
 import types
 
-from vyper.compiler.settings import (
-    VYPER_ERROR_CONTEXT_LINES,
-    VYPER_ERROR_LINE_NUMBERS,
-)
+from vyper.compiler.settings import VYPER_ERROR_CONTEXT_LINES, VYPER_ERROR_LINE_NUMBERS
 
 
 class ExceptionList(list):
@@ -144,6 +141,12 @@ class SyntaxException(VyperException):
         super().__init__(message, item)
 
 
+class DecimalOverrideException(VyperException):
+    """The Vyper compiler uses specific Decimal settings which
+    if overridden could lead to incorrect behavior.
+    """
+
+
 class NatSpecSyntaxException(SyntaxException):
     """Invalid syntax within NatSpec docstring."""
 
