@@ -1,13 +1,11 @@
 from typing import Optional
 
 from vyper import ast as vy_ast
-from vyper.ast.signatures.function_signature import (
-    ContractRecord,
-    VariableRecord,
-)
+from vyper.ast.signatures.function_signature import ContractRecord, VariableRecord
 from vyper.exceptions import CompilerPanic, InvalidType, StructureException
 from vyper.old_codegen.types import InterfaceType, parse_type
 from vyper.typing import InterfaceImports
+from vyper.utils import cached_property
 
 
 # Datatype to store all global context information.
@@ -32,10 +30,7 @@ class GlobalContext:
         cls, vyper_module: "vy_ast.Module", interface_codes: Optional[InterfaceImports] = None
     ) -> "GlobalContext":
         # TODO is this a cyclic import?
-        from vyper.ast.signatures.interface import (
-            extract_sigs,
-            get_builtin_interfaces,
-        )
+        from vyper.ast.signatures.interface import extract_sigs, get_builtin_interfaces
 
         interface_codes = {} if interface_codes is None else interface_codes
         global_ctx = cls()
@@ -120,9 +115,7 @@ class GlobalContext:
                 # A struct must be defined before it is referenced.
                 # This feels like a semantic step and maybe should be pushed
                 # to a later compilation stage.
-                parse_type(
-                    member_type, "storage", custom_structs=self._structs,
-                )
+                self.parse_type(member_type, "storage")
                 members.append((member_name, member_type))
             else:
                 raise StructureException("Structs can only contain variables", item)
@@ -200,26 +193,50 @@ class GlobalContext:
                     f" Use {item.target.id}: {item_name} instead"
                 )
             self._globals[item.target.id] = ContractRecord(
-                item.target.id, len(self._globals), InterfaceType(item_name), True,
+                item.target.id,
+                len(self._globals),
+                InterfaceType(item_name),
+                True,
             )
         elif self.get_call_func_name(item) == "public":
             if isinstance(item.annotation.args[0], vy_ast.Name) and item_name in self._contracts:
                 typ = InterfaceType(item_name)
             else:
-                typ = parse_type(item.annotation.args[0], "storage", custom_structs=self._structs,)
+                typ = self.parse_type(item.annotation.args[0], "storage")
             self._globals[item.target.id] = VariableRecord(
-                item.target.id, len(self._globals), typ, True,
+                item.target.id,
+                len(self._globals),
+                typ,
+                True,
+            )
+        elif self.get_call_func_name(item) == "immutable":
+            typ = self.parse_type(item.annotation.args[0], "code")
+            self._globals[item.target.id] = VariableRecord(
+                item.target.id,
+                len(self._globals),
+                typ,
+                False,
+                is_immutable=True,
             )
 
         elif isinstance(item.annotation, (vy_ast.Name, vy_ast.Call, vy_ast.Subscript)):
             self._globals[item.target.id] = VariableRecord(
                 item.target.id,
                 len(self._globals),
-                parse_type(item.annotation, "storage", custom_structs=self._structs,),
+                self.parse_type(item.annotation, "storage"),
                 True,
             )
         else:
             raise InvalidType("Invalid global type specified", item)
 
-    def parse_type(self, ast_node, location):
-        return parse_type(ast_node, location, sigs=self._contracts, custom_structs=self._structs,)
+    def parse_type(self, ast_node, location=None):
+        return parse_type(
+            ast_node,
+            location,
+            sigs=self._contracts,
+            custom_structs=self._structs,
+        )
+
+    @cached_property
+    def immutable_section_size(self):
+        return sum([imm.size * 32 for imm in self._globals.values() if imm.is_immutable])
