@@ -12,6 +12,8 @@ CONSTANT_ENVIRONMENT_VARS: Dict[str, Dict[str, type]] = {
         "coinbase": AddressDefinition,
         "difficulty": Uint256Definition,
         "number": Uint256Definition,
+        "gaslimit": Uint256Definition,
+        "basefee": Uint256Definition,
         "prevhash": Bytes32Definition,
         "timestamp": Uint256Definition,
     },
@@ -37,8 +39,8 @@ def get_constant_vars() -> Dict:
     """
     result = {}
     for name, members in CONSTANT_ENVIRONMENT_VARS.items():
-        members = {k: v(is_immutable=True) for k, v in members.items()}
-        result[name] = StructDefinition(name, members, is_immutable=True)
+        members = {k: v(is_constant=True) for k, v in members.items()}
+        result[name] = StructDefinition(name, members, is_constant=True)
 
     return result
 
@@ -48,8 +50,4 @@ def get_mutable_vars() -> Dict:
     Get a dictionary of mutable environment variables (those that are
     modified during the course of contract execution, such as `self`).
     """
-    result = {}
-    for name, type_ in MUTABLE_ENVIRONMENT_VARS.items():
-        result[name] = type_(is_immutable=True)
-
-    return result
+    return {name: type_(is_constant=True) for name, type_ in MUTABLE_ENVIRONMENT_VARS.items()}
