@@ -1,5 +1,7 @@
 import binascii
 import functools
+import sys
+import traceback
 from typing import Dict, List, Union
 
 from vyper.exceptions import InvalidLiteral
@@ -13,12 +15,37 @@ except ImportError:
 
     keccak256 = lambda x: _sha3.sha3_256(x).digest()  # noqa: E731
 
+try:
+    # available py3.8+
+    from functools import cached_property
+except ImportError:
+    from cached_property import cached_property  # type: ignore
+
 
 # Converts four bytes to an integer
 def fourbytes_to_int(inp):
     return (inp[0] << 24) + (inp[1] << 16) + (inp[2] << 8) + inp[3]
 
 
+# utility function for debugging purposes
+def trace(n=5, out=sys.stderr):
+    print("BEGIN TRACE", file=out)
+    for x in list(traceback.format_stack())[-n:]:
+        print(x.strip(), file=out)
+    print("END TRACE", file=out)
+
+
+# converts a signature like Func(bool,uint256,address) to its 4 byte method ID
+# TODO replace manual calculations in codebase with this
+def abi_method_id(method_sig):
+    return fourbytes_to_int(keccak256(bytes(method_sig, "utf-8"))[:4])
+
+
+# map a string to only-alphanumeric chars
+def mkalphanum(s):
+    return "".join([c if c.isalnum() else "_" for c in s])
+
+
 # Converts string to bytes
 def string_to_bytes(str):
     bytez = b""
@@ -71,24 +98,22 @@ def calc_mem_gas(memsize):
 # Specific gas usage
 GAS_IDENTITY = 15
 GAS_IDENTITYWORD = 3
+GAS_CODECOPY_WORD = 3
+GAS_CALLDATACOPY_WORD = 3
 
 # A decimal value can store multiples of 1/DECIMAL_DIVISOR
 MAX_DECIMAL_PLACES = 10
 DECIMAL_DIVISOR = 10 ** MAX_DECIMAL_PLACES
 
 
-# Number of bytes in memory used for system purposes, not for variables
+# memory used for system purposes, not for variables
 class MemoryPositions:
-    ADDRSIZE = 32
-    MAX_INT128 = 64
-    MIN_INT128 = 96
-    MAXDECIMAL = 128
-    MINDECIMAL = 160
-    FREE_VAR_SPACE = 192
-    FREE_VAR_SPACE2 = 224
-    BLANK_SPACE = 256
-    FREE_LOOP_INDEX = 288
-    RESERVED_MEMORY = 320
+    MAXDECIMAL = 32
+    MINDECIMAL = 64
+    FREE_VAR_SPACE = 128
+    FREE_VAR_SPACE2 = 160
+    FREE_LOOP_INDEX = 192
+    RESERVED_MEMORY = 224
 
 
 # Sizes of different data types. Used to clamp types.
@@ -100,6 +125,7 @@ class SizeLimits:
     MIN_INT256 = -(2 ** 255)
     MAXDECIMAL = (2 ** 127 - 1) * DECIMAL_DIVISOR
     MINDECIMAL = (-(2 ** 127)) * DECIMAL_DIVISOR
+    MAX_UINT8 = 2 ** 8 - 1
     MAX_UINT256 = 2 ** 256 - 1
 
     @classmethod
@@ -107,7 +133,9 @@ class SizeLimits:
         assert isinstance(type_str, str)
         if type_str == "decimal":
             return float(cls.MINDECIMAL) <= value <= float(cls.MAXDECIMAL)
-        if type_str == "uint256":
+        if type_str == "uint8":
+            return 0 <= value <= cls.MAX_UINT8
+        elif type_str == "uint256":
             return 0 <= value <= cls.MAX_UINT256
         elif type_str == "int128":
             return cls.MIN_INT128 <= value <= cls.MAX_INT128
@@ -120,9 +148,6 @@ class SizeLimits:
 # Map representing all limits loaded into a contract as part of the initializer
 # code.
 LOADED_LIMITS: Dict[int, int] = {
-    MemoryPositions.ADDRSIZE: SizeLimits.ADDRSIZE,
-    MemoryPositions.MAX_INT128: SizeLimits.MAX_INT128,
-    MemoryPositions.MIN_INT128: SizeLimits.MIN_INT128,
     MemoryPositions.MAXDECIMAL: SizeLimits.MAXDECIMAL,
     MemoryPositions.MINDECIMAL: SizeLimits.MINDECIMAL,
 }
@@ -172,7 +197,7 @@ VALID_LLL_MACROS = {
 }
 
 # Available base types
-BASE_TYPES = {"int128", "int256", "decimal", "bytes32", "uint256", "bool", "address"}
+BASE_TYPES = {"int128", "int256", "decimal", "bytes32", "uint8", "uint256", "bool", "address"}
 
 
 def is_instances(instances, instance_type):
@@ -260,7 +285,7 @@ def annotate_source_code(
         mark_repr = "-" * col_offset + "^" + "\n"
 
     before_lines = "".join(source_lines[start_offset:line_offset])
-    after_lines = "".join(source_lines[line_offset + 1 : end_offset])  # noqa: E203
+    after_lines = "".join(source_lines[line_offset + 1 : end_offset])
     location_repr = "".join((before_lines, line_repr, mark_repr, after_lines))
 
     if line_numbers:
@@ -293,3 +318,8 @@ def annotate_source_code(
     cleanup_lines += [""] * (num_lines - len(cleanup_lines))
 
     return "\n".join(cleanup_lines)
+
+
+__all__ = [
+    "cached_property",
+]
